@@ -1,13 +1,20 @@
 from __future__ import annotations
 
+import asyncio
+import time
 from functools import partial
-from typing import Type, Generic, Callable
+from types import SimpleNamespace
+from typing import Type, Generic, Callable, Dict, Any
 
+import loguru
 import orjson
-from sanic import Sanic
+from sanic import Sanic, Request
+from sanic.response import BaseHTTPResponse
 
 from web_foundation.config import AppConf
 from web_foundation.extentions.error_handler import ExtendedErrorHandler
+from web_foundation.extentions.metrics.events import NewMetricEvent
+from web_foundation.extentions.metrics.metric import HistogramMetric, CounterMetric, SummaryMetric, Metric
 from web_foundation.extentions.realtime.broadcaster import RtBroadcaster
 from web_foundation.extentions.realtime.connection import RtEventCallback
 from web_foundation.extentions.router import ExtRouter
@@ -16,6 +23,7 @@ from web_foundation.kernel.messaging.channel import IChannel, IMessage, GenericI
 from web_foundation.kernel.container import GenericDependencyContainer
 from web_foundation.kernel.messaging.dispatcher import IDispatcher, IDispatcherIsolate
 from web_foundation.kernel.isolate import Isolate
+from web_foundation import settings
 
 
 class App(Generic[GenericDependencyContainer]):
@@ -37,7 +45,7 @@ class App(Generic[GenericDependencyContainer]):
 
         self.dispatcher = dispatcher if dispatcher else IDispatcher()
         self._rt_listeners = []
-        self.add_isolate("Dispatcher", IDispatcherIsolate, dispatcher=self.dispatcher, add_channel=False, daemon=False)
+        self.add_isolate("Dispatcher", IDispatcherIsolate, dispatcher=self.dispatcher, add_channel=False, daemon=True)
 
     def add_isolate(self, name: str, isolate_cls: Type[Isolate], daemon=True, add_channel: bool = True, **kwargs):
         """
@@ -50,7 +58,7 @@ class App(Generic[GenericDependencyContainer]):
         def add_nondaemon(s_app: Sanic):
             nonlocal instance
 
-            def override_manage(ident, func, kwargs, transient=False, daemon=True):
+            def override_manage(ident, func, kwargs: Dict[str, Any], transient=False, daemon=True):
                 nonlocal s_app
                 container = s_app.manager.transient if transient else s_app.manager.durable
                 container.append(
@@ -62,6 +70,7 @@ class App(Generic[GenericDependencyContainer]):
         if daemon:
             self.sanic.main_process_ready(
                 lambda s_app: s_app.manager.manage(name, instance.run_forked, {}))
+            # self.sanic.main_process_ready(lambda s_app:s_app.manager.)
         else:
             self.sanic.main_process_ready(lambda s_app: add_nondaemon(s_app))
 
@@ -98,7 +107,6 @@ class App(Generic[GenericDependencyContainer]):
 
     def _configure_sanic_worker(self):
         async def set_worker_ctx(s_app: Sanic):
-
             # set channel to ctx
             worker_num = int(s_app.m.name.split('-')[-2])
             s_app.ctx.channel = getattr(s_app.shared_ctx, f"Channel_{worker_num}")
@@ -116,7 +124,44 @@ class App(Generic[GenericDependencyContainer]):
                     use_nested_classes=use_nested_classes)
             await self.container.init_resources(s_app.ctx.channel)
 
+        async def reg_metrics(s_app: Sanic):
+            async def give_metric(metric: Metric):
+                asyncio.create_task(s_app.ctx.channel.produce(NewMetricEvent(metric)))
+
+            self.sanic.ctx.metrics = SimpleNamespace()
+
+            async def metrics_before(request: Request):
+                if not settings.METRICS_ENABLE:
+                    return
+                request.app.ctx.metrics.exec_time = time.time()
+
+            async def metrics_after(request: Request, response: BaseHTTPResponse):
+                if not settings.METRICS_ENABLE:
+                    return
+                else:
+                    metr = HistogramMetric(f"exec_request_time")
+                    metr.observe(time.time() - request.app.ctx.metrics.exec_time)
+                    metr.add_label(request=request.path,
+                                   method=request.method,
+                                   status=str(response.status))
+                    await give_metric(metr)
+                    metr = CounterMetric(f"request_path_count", value=1)
+                    metr.add_label(request=request.path,
+                                   method=request.method,
+                                   status=str(response.status))
+                    await give_metric(metr)
+                    metr = SummaryMetric(f"request_path_latency")
+                    metr.add_label(request=request.path,
+                                   method=request.method,
+                                   status=str(response.status))
+                    metr.observe(time.time() - request.app.ctx.metrics.exec_time)
+                    await give_metric(metr)
+
+            self.sanic.register_middleware(metrics_before)
+            self.sanic.register_middleware(metrics_after, "response")
+
         self.sanic.before_server_start(set_worker_ctx)
+        self.sanic.before_server_start(reg_metrics)
 
         async def listen_channel(s_app: Sanic):
             # noinspection PyAsyncCall
