@@ -1,14 +1,20 @@
 import asyncio
+import signal
 from abc import ABCMeta
 from typing import Dict, Generic
 
+import loguru
+from sanic import Sanic
+from sanic.worker.multiplexer import WorkerMultiplexer
+
+from web_foundation.extentions.sanic_ext_worker import ExtWorker
 from web_foundation.kernel.container import GenericDependencyContainer
 from web_foundation.kernel.messaging.channel import IChannel
 
 
 class Isolate(Generic[GenericDependencyContainer], metaclass=ABCMeta):
     name: str
-    channel: IChannel
+    channel: IChannel | None
     container: GenericDependencyContainer
     init_kwargs: Dict
 
@@ -29,6 +35,12 @@ class Isolate(Generic[GenericDependencyContainer], metaclass=ABCMeta):
             await self.work(**self.init_kwargs)
             await self.container.shutdown_resources()
 
+        def signal_handler(sig, frame):
+            for task in asyncio.all_tasks():
+                task.cancel()
+
+        signal.signal(signal.SIGINT, signal_handler)
+
         try:
             asyncio.run(call())
         except KeyboardInterrupt:
