@@ -12,22 +12,29 @@ from web_foundation.kernel.isolate import Isolate
 from web_foundation.extentions.schedulle.scheduller import TaskScheduler, TaskIMessage
 from web_foundation.settings_event import SettingsChangeEvent
 
+MetricsDict = Dict[str, Dict[str, Metric]]
+
 
 class IDispatcher:
     channels: Dict[str, IChannel]
     events_listeners: Dict[str, EventListener]
     scheduler: TaskScheduler
-    collected_metrics: Dict[str, Dict[str, Metric]]
+    collected_metrics: Dict[str, MetricsDict]
     _msg_global_index: int
+    state: Dict[str, Any]
 
     def __init__(self, scheduler: TaskScheduler = None):
         self.scheduler = scheduler
+        self.state = {}
         self.channels = {}
         self.events_listeners = {}
         self.collected_metrics = {}
+        self.collected_metrics.update({"__system_metric__": {}})
         self._msg_global_index = 0
         if self.scheduler:
             self.scheduler.produce_method = self.broadcast
+            self.state.update({"scheduler": {}})
+            self.scheduler.state = self.state.get("scheduler")
             self.add_event_listener(TaskIMessage.message_type, self.scheduler.add_task)
         if settings.METRICS_ENABLE:
             self.add_event_listener(NewMetricEvent.message_type, self.on_new_metric)
@@ -51,14 +58,15 @@ class IDispatcher:
 
         if settings.METRICS_ENABLE:
             event_counter = CounterMetric("events_counter", self._msg_global_index)
-            self.collected_metrics.update({"events_counter": {"dispatcher_events_counter": event_counter}})
+            save_to = self.collected_metrics['__system_metric__']
+            save_to.update({"events_counter": {"dispatcher_events_counter": event_counter}})
 
-            named_event_counter = self.collected_metrics.get("named_events_counter")
+            named_event_counter = save_to.get("named_events_counter")
             if not named_event_counter:
                 mtr = CounterMetric("named_events_counter", value=1)
                 mtr.add_label(event_name=msg.message_type)
                 named_event_counter = {msg.message_type: mtr}
-                self.collected_metrics["named_events_counter"] = named_event_counter
+                save_to["named_events_counter"] = named_event_counter
             elif named_event_counter.get(msg.message_type):
                 named_event_counter[msg.message_type].inc()
             else:
@@ -81,10 +89,14 @@ class IDispatcher:
         if sender_channel:
             if settings.DEBUG:
                 loguru.logger.debug(f"Send METRICS to {sender_channel}")
-            if not self.collected_metrics:
-                await sender_channel.sent_to_consume(exporter.empty())
+            if not self.collected_metrics.get(event.kind) or not self.collected_metrics[
+                event.kind] or not settings.METRICS_ENABLE:
+                resp = MetricResponseEvent(exporter.empty())
+                resp.inner_index = event.inner_index
+                await sender_channel.sent_to_consume(resp)
+                return
             rp = []
-            for i in self.collected_metrics.values():
+            for i in self.collected_metrics[event.kind].values():
                 if isinstance(i, dict):
                     for k in i.values():
                         rp.append(k)
@@ -96,9 +108,12 @@ class IDispatcher:
             await sender_channel.sent_to_consume(resp)
 
     async def on_new_metric(self, event: NewMetricEvent):
-        metr = self.collected_metrics.get(event.metric.name)
+        if not self.collected_metrics.get(event.metric.kind):
+            self.collected_metrics.update({event.metric.kind: {}})
+        metr = self.collected_metrics[event.metric.kind].get(event.metric.name)
         if not metr:
-            self.collected_metrics.update({event.metric.name: {event.metric.labels_concat: event.metric}})
+            self.collected_metrics[event.metric.kind].update(
+                {event.metric.name: {event.metric.labels_concat: event.metric}})
         else:
             if metr.get(event.metric.labels_concat):
                 # loguru.logger.warning(f"get {event.metric}")
